# Copyright (C) 2011 Adam Stein <adam@csh.rit.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
Provide utilities for printing endnotes in text reports.
"""

from gen.plug.report import endnotes

def add_endnote_styles(style_sheet):
    endnotes.add_endnote_styles(style_sheet)

def cite_source(bibliography, obj):
    return endnotes.cite_source(bibliography, obj)

def write_endnotes(bibliography, database, doc, printnotes=False, links=False):
    """
    Write all the entries in the bibliography as endnotes.

    @param bibliography: The bibliography that contains the citations.
    @type bibliography: L{Bibliography}
    @param database: The database that the sources come from.
    @type database: DbBase
    @param doc: The document to write the endnotes into.
    @type doc: L{docgen.TextDoc}
    @param printnotes: Indicate if the notes attached to a source must be
        written too.
    @type printnotes: bool
    """
    if bibliography.get_citation_count() == 0:
        return

    doc.start_paragraph('Endnotes-Header')
    doc.write_text(_('Endnotes'))
    doc.end_paragraph()

    cindex = 0
    for citation in bibliography.get_citation_list():
        cindex += 1
        source = database.get_source_from_handle(citation.get_source_handle())
        first = True

        doc.start_paragraph('Endnotes-Source', "%d." % cindex)

        src_txt = endnotes._format_source_text(source)

        doc.write_text(src_txt, links=links)
        doc.end_paragraph()

        ref_list = citation.get_ref_list()

        if ref_list:
            first = True
            reflines = ""
            for key, ref in ref_list:
                txt = "%s: %s" % (key, ref.get_page())
                if first:
                    reflines += txt
                    first = False
                else:
                    reflines += ('\n%s' % txt)
                doc.write_endnotes_ref(reflines,'Endnotes-Ref', links=links)

        if printnotes:
            note_list = source.get_note_list()
            ind = 1
            for notehandle in note_list:
                note = database.get_note_from_handle(notehandle)
                doc.start_paragraph('Endnotes-Notes')
                doc.write_text(_('Note %(ind)d - Type: %(type)s') % {
                                'ind': ind,
                                'type': str(note.get_type())})
                doc.end_paragraph()
                doc.write_styled_note(note.get_styledtext(),
                                        note.get_format(),'Endnotes-Notes',
                                        contains_html= note.get_type() \
                                                        == endnotes.NoteType.HTML_CODE, links=links)
                ind += 1

