# Copyright (C) 2011 Adam Stein <adam@csh.rit.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#-------------------------------------------------------------------------
#
# Optional Python Modules 
#
#-------------------------------------------------------------------------
try:
    import PIL.Image

    have_PIL = True
except ImportError:
    have_PIL = False

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------
import ImgManip

#-------------------------------------------------------------------------
#
# ImgManip_Enhanced
#
#-------------------------------------------------------------------------
class ImgManip_Enhanced(object):
    """
    The Image Manipulation class
    """

    def __init__(self, filename):
        if have_PIL:
            try:
                self.img = PIL.Image.open(filename)
            except IOError:
                self.img = None
        else:
            import gtk
            import gobject

            try:
                self.img = gtk.gdk.pixbuf_new_from_file(filename)
            except gobject.GError:
                self.img = None

    def image_dpi(self):
        """
        Return the dpi found in the image header.  None is returned if no dpi attribute
        is available.
        """

        try:
            return self.img.info["dpi"]
        except AttributeError, KeyError:
            return None

    def image_size(self):
        """
        Return the width and size of the image.

        :rtype: tuple(int, int)
        :returns: a tuple consisting of the width and height
        """

        if self.img:
            if have_PIL:
                size = self.img.size
            else:
                size = (self.img.get_width(), self.img.get_height())
        else:
            size = (0, 0)

        return size

    def resize_to_jpeg(self, destination, width, height):
        """
        Create the destination, derived from the image, resizing it to the
        specified size, while converting to JPEG.

        :param destination: destination image file, output written in jpeg format
        :type destination: unicode
        :param width: desired width of the destination image
        :type width: int
        :param height: desired height of the destination image
        :type height: int
        """

        scaled = self.img.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
        scaled.save(destination, "jpeg")

    def resize_to_jpeg_buffer(source, width, height):
            """
            Convert the image to JPEG and resize it. Instead of
            saving the file, the data is returned in a buffer.

            :param width: desired width of the destination image
            :type width: int
            :param height: desired height of the destination image
            :type height: int
            :rtype: buffer of data
            :returns: jpeg image as raw data
            """

            dest = tempfile.mkstemp()

            scaled = self.img.scale_simple(int(width), int(height), gtk.gdk.INTERP_BILINEAR)

            dest = Utils.get_unicode_path_from_env_var(dest)
            scaled.save(dest, "jpeg")

            ofile = open(dest, mode="rb")
            data = ofile.read()
            ofile.close()

            try:
                os.unlink(dest)
            except:
                pass

            return data

